'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = enableGracefulClose;

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var log = (0, _debug2.default)('graceful');

function enableGracefulClose(server, userOptions, callback) {
  if (typeof userOptions === 'function') {
    return enableGracefulClose(server, {}, userOptions);
  }

  var options = _extends({
    timeout: 10 * 1000 }, userOptions);

  var connections = new Set();
  var closing = false;

  server.on('connection', function (connection) {
    connections.add(connection);
    connection.on('close', function () {
      return connections.delete(connection);
    });
  });

  function close() {
    var _this = this;

    log('Graceful close start');
    if (closing) {
      log('Server closing is already in the progress');
      return;
    }

    closing = true;

    // without all closed connections close will not call callback
    if (connections.size) {
      log('We have keep-alive connections. Waiting');
      setTimeout(function () {
        log('Closing all connections');
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = connections[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var connection = _step.value;

            connection.end();
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        connections.clear();
      }, options.timeout);
    }

    // server can be opened and closed again
    log('Closing listening');
    server.close(_asyncToGenerator(regeneratorRuntime.mark(function _callee() {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              log('Server listening is closed');
              closing = false;

              // end process

              if (!callback) {
                _context.next = 5;
                break;
              }

              _context.next = 5;
              return callback();

            case 5:

              process.exit(0);

            case 6:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    })));
  }

  process.on('SIGINT', close);

  return close;
}